package devguide.model;
import devguide.model.common.SRService;
import devguide.model.entities.ProductImpl;
import devguide.model.entities.ServiceRequestImpl;
import devguide.model.entities.UserImpl;
import devguide.model.queries.ServiceRequestsImpl;

import oracle.jbo.Key;
import oracle.jbo.server.ApplicationModuleImpl;
import oracle.jbo.server.EntityDefImpl;
import oracle.jbo.server.EntityImpl;
import oracle.jbo.*;
import oracle.jbo.client.Configuration;
import oracle.jbo.domain.*;
import oracle.jbo.server.ViewLinkImpl;
import oracle.jbo.server.ViewObjectImpl;
// ---------------------------------------------------------------------
// ---    File generated by Oracle ADF Business Components Design Time.
// ---    Custom code may be added to this class.
// ---    Warning: Do not modify method signatures of generated methods.
// ---------------------------------------------------------------------
 /**
  * This custom application module class illustrates the same
  * example methods as from the SRServiceImpl2.java in the
  * BusinessLayerWithEntityObjects project, using the strongly typed
  * access to the entity objects each of whose custom Java classes
  * has been generated.
  */

public class SRServiceImpl extends ApplicationModuleImpl implements SRService {
  /**This is the default constructor (do not remove)
   */
  public SRServiceImpl() {
  }
  /*
   * Helper method to return a ServiceRequest by Id
   */
  private ServiceRequestImpl retrieveServiceRequestById(long requestId) {
    EntityDefImpl svcReqDef = ServiceRequestImpl.getDefinitionObject();
    Key svcReqKey = ServiceRequestImpl.createPrimaryKey(new DBSequence(requestId));
    return (ServiceRequestImpl)svcReqDef.findByPrimaryKey(getDBTransaction(),svcReqKey);
  }

  /*
   * Find a ServiceRequest by Id
   */
  public String findServiceRequestStatus(long requestId) {
    ServiceRequestImpl svcReq = retrieveServiceRequestById(requestId);
    if (svcReq != null) {
      return svcReq.getStatus();
    }
    return null;
  }
  
  /*
   * Create a new Product and Return its new id
   */
  public long createProduct(String name, String description) {
    EntityDefImpl productDef = ProductImpl.getDefinitionObject();
    ProductImpl newProduct = (ProductImpl)productDef.createInstance2(getDBTransaction(),null);
    newProduct.setName(name);
    newProduct.setDescription(description);
    try {
      getDBTransaction().commit();
    }
    catch (JboException ex) {
      getDBTransaction().rollback();
      throw ex;
    }
    DBSequence newIdAssigned = newProduct.getProdId();
    return newIdAssigned.getSequenceNumber().longValue();
  }
  /*
   * Update the status of an existing service request
   */  
  public void updateRequestStatus(long requestId, String newStatus) {
    ServiceRequestImpl svcReq = retrieveServiceRequestById(requestId);
    if (svcReq != null) {
      svcReq.setStatus(newStatus);
      try {
        getDBTransaction().commit();
      }
      catch (JboException ex) {
        getDBTransaction().rollback();
        throw ex;
      }
    }
  }
  
  /*
   * Access an associated Used entity from the ServiceRequest entity
   */  
  public String findServiceRequestTechnician(long requestId) {
    ServiceRequestImpl svcReq = retrieveServiceRequestById(requestId);
    if (svcReq != null) {
      UserImpl tech  = (UserImpl)svcReq.getTechnicianAssigned();
      if (tech != null) {
        return tech.getFirstName()+" "+tech.getLastName();
      }
      else {
        return "Unassigned";
      }
    }
    else {
      return null;
    }
  }  
   /*
    * Testing method
    */
//    public static void main(String[] args) {
//      String        amDef = "devguide.model.SRService";
//      String        config = "SRServiceLocal";
//      ApplicationModule am = Configuration.createRootApplicationModule(amDef,config);
//      /* 
//       * NOTE: This cast to use the SRServiceImpl class is OK since this
//       * ----  code is inside a business tier *Impl.java file and not in a
//       *       client class that is accessing the business tier from "outside".
//       */
//      SRServiceImpl service = (SRServiceImpl)am;
//      String status = service.findServiceRequestStatus(101);
//      System.out.println("Status of SR# 101 = " + status);
//      String techName = service.findServiceRequestTechnician(101);
//      System.out.println("Technician for SR# 101 = " + techName);
//      try {
//        service.updateRequestStatus(101,"Reopened");
//      }
//      catch (JboException ex) {
//        System.out.println("ERROR: "+ex.getMessage());
//      }
//      long id = 0;
//      try {
//        id = service.createProduct(null,"Makes Blended Fruit Drinks");
//      }
//      catch (JboException ex) {
//        System.out.println("ERROR: "+ex.getMessage());
//      }
//      id = service.createProduct("Smoothie Maker","Makes Blended Fruit Drinks");
//      System.out.println("New product created successfully with id = "+id);
//      Configuration.releaseRootApplicationModule(am,true);
//    }

 //Original main() method generated by the application module editor
  /**Sample main for debugging Business Components code using the tester.
   */
  public static void main(String[] args) {
    launchTester("devguide.model", /* package name */
      "SRServiceLocal" /* Configuration Name */);
  }


  /**Container's getter for StaffList
   */
  public ViewObjectImpl getStaffList() {
    return (ViewObjectImpl)findViewObject("StaffList");
  }

  /**Container's getter for ServiceRequests
   */
  public ServiceRequestsImpl getServiceRequests() {
    return (ServiceRequestsImpl)findViewObject("ServiceRequests");
  }

  /**Container's getter for AssignedServiceRequests
   */
  public ServiceRequestsImpl getAssignedServiceRequests() {
    return (ServiceRequestsImpl)findViewObject("AssignedServiceRequests");
  }

  /**Container's getter for AssignedServiceHistories
   */
  public ViewObjectImpl getAssignedServiceHistories() {
    return (ViewObjectImpl)findViewObject("AssignedServiceHistories");
  }

  /**Container's getter for ServiceHistories
   */
  public ViewObjectImpl getServiceHistories() {
    return (ViewObjectImpl)findViewObject("ServiceHistories");
  }

  /**Container's getter for RequestsAssignedToTechnician1
   */
  public ViewLinkImpl getRequestsAssignedToTechnician1() {
    return (ViewLinkImpl)findViewLink("RequestsAssignedToTechnician1");
  }

  /**Container's getter for HistoryLinesForRequest1
   */
  public ViewLinkImpl getHistoryLinesForRequest1() {
    return (ViewLinkImpl)findViewLink("HistoryLinesForRequest1");
  }

  /**Container's getter for HistoryLinesForRequest2
   */
  public ViewLinkImpl getHistoryLinesForRequest2() {
    return (ViewLinkImpl)findViewLink("HistoryLinesForRequest2");
  }

  /**Container's getter for OpenProblemsAndAssignees
   */
  public ViewObjectImpl getOpenProblemsAndAssignees() {
    return (ViewObjectImpl)findViewObject("OpenProblemsAndAssignees");
  }
  protected void prepareSession(Session session) {
    super.prepareSession(session);
    findLoggedInUserByEmailInStaffList();
  }
  private void findLoggedInUserByEmailInStaffList() {
    ViewObject staffList = getStaffList();
     String currentUserName = getUserPrincipalName();
     /*
      * Until later when we learn how to integrate J2EE security,
      * this API will return null. For testing, we can default it
      * to the email of one of the staff members like "sking".
      */
     if (currentUserName == null) {
       currentUserName = "sking";
     }
    /*
     * We can't use a simple findByKey since the key for the
     * StaffList view object is the numerical userid of the staff member
     * and we want to find the user by their email address. We could build
     * an "EMAIL = :CurrentUser" where clause directly into the view object
     * at design time, but here let's illustrate doing it dynamically to
     * see this alternative.
     */
    staffList.defineNamedWhereClauseParam("CurrentUser",    // bindvar name
                                           currentUserName, // default value
                                           null);
    staffList.setWhereClause("EMAIL = :CurrentUser");
    staffList.executeQuery();
    /*
     * If the view object needs to be also used during this session 
     * without the additional where clause, you would use
     * setWhereClause(null) and removeNamedWhereClauseParam("CurrentUser") to
     * leave the view object instance back in it's original state.
     */
  }
}
