package devguide.model;
import devguide.model.entities.ProductImpl;
import devguide.model.entities.ServiceRequestImpl;
import devguide.model.entities.UserImpl;

import oracle.jbo.ApplicationModule;
import oracle.jbo.JboException;
import oracle.jbo.Key;
import oracle.jbo.client.Configuration;
import oracle.jbo.domain.DBSequence;
import oracle.jbo.domain.Number;
import oracle.jbo.server.ApplicationModuleImpl;
import oracle.jbo.server.EntityDefImpl;
import oracle.jbo.server.EntityImpl;
// ---------------------------------------------------------------------
// ---    File generated by Oracle ADF Business Components Design Time.
// ---    Custom code may be added to this class.
// ---    Warning: Do not modify method signatures of generated methods.
// ---------------------------------------------------------------------
/**
 * This custom application module class illustrates the same
 * example methods as SRServiceImpl.java, except that here
 * we're using the strongly-typed custom Entity Java classes
 * ServiceRequestImpl, UserImpl, and ProductImpl instead of working
 * with all the entity objects using the base EntityImpl class.
 */
public class SRService2Impl extends ApplicationModuleImpl {
  /**This is the default constructor (do not remove)
   */
  public SRService2Impl() {
  }
  /*
   * Helper method to return a ServiceRequest by Id
   */
  private ServiceRequestImpl retrieveServiceRequestById(long requestId) {
    EntityDefImpl svcReqDef = ServiceRequestImpl.getDefinitionObject();
    Key svcReqKey = ServiceRequestImpl.createPrimaryKey(new DBSequence(requestId));
    return (ServiceRequestImpl)svcReqDef.findByPrimaryKey(getDBTransaction(),svcReqKey);
  }

  /*
   * Find a ServiceRequest by Id
   */
  public String findServiceRequestStatus(long requestId) {
    ServiceRequestImpl svcReq = retrieveServiceRequestById(requestId);
    if (svcReq != null) {
      return svcReq.getStatus();
    }
    return null;
  }
  
  /*
   * Create a new Product and Return its new id
   */
  public long createProduct(String name, String description) {
    EntityDefImpl productDef = ProductImpl.getDefinitionObject();
    ProductImpl newProduct = (ProductImpl)productDef.createInstance2(getDBTransaction(),null);
    newProduct.setName(name);
    newProduct.setDescription(description);
    try {
      getDBTransaction().commit();
    }
    catch (JboException ex) {
      getDBTransaction().rollback();
      throw ex;
    }
    DBSequence newIdAssigned = newProduct.getProdId();
    return newIdAssigned.getSequenceNumber().longValue();
  }
  /*
   * Update the status of an existing service request
   */  
  public void updateRequestStatus(long requestId, String newStatus) {
    ServiceRequestImpl svcReq = retrieveServiceRequestById(requestId);
    if (svcReq != null) {
      svcReq.setStatus(newStatus);
      try {
        getDBTransaction().commit();
      }
      catch (JboException ex) {
        getDBTransaction().rollback();
        throw ex;
      }
    }
  }
  
  /*
   * Access an associated Used entity from the ServiceRequest entity
   */  
  public String findServiceRequestTechnician(long requestId) {
    ServiceRequestImpl svcReq = retrieveServiceRequestById(requestId);
    if (svcReq != null) {
      UserImpl tech  = (UserImpl)svcReq.getTechnicianAssigned();
      if (tech != null) {
        return tech.getFirstName()+" "+tech.getLastName();
      }
      else {
        return "Unassigned";
      }
    }
    else {
      return null;
    }
  }  
  // Original main() method generated by the application module editor
  //
  //  /**Sample main for debugging Business Components code using the tester.
  //   */
  //  public static void main(String[] args) {
  //    launchTester("devguide.model", /* package name */
  //      "SRServiceLocal" /* Configuration Name */);
  //  }
   /*
    * Testing method
    */
    public static void main(String[] args) {
      String        amDef = "devguide.model.SRService";
      String        config = "SRServiceLocal";
      ApplicationModule am = Configuration.createRootApplicationModule(amDef,config);
      /* 
       * NOTE: This cast to use the SRServiceImpl class is OK since this
       * ----  code is inside a business tier *Impl.java file and not in a
       *       client class that is accessing the business tier from "outside".
       */
      SRServiceImpl service = (SRServiceImpl)am;
      String status = service.findServiceRequestStatus(101);
      System.out.println("Status of SR# 101 = " + status);
      String techName = service.findServiceRequestTechnician(101);
      System.out.println("Technician for SR# 101 = " + techName);
      try {
        service.updateRequestStatus(101,"Reopened");
      }
      catch (JboException ex) {
        System.out.println("ERROR: "+ex.getMessage());
      }
      long id = 0;
      try {
        id = service.createProduct(null,"Makes Blended Fruit Drinks");
      }
      catch (JboException ex) {
        System.out.println("ERROR: "+ex.getMessage());
      }
      id = service.createProduct("Smoothie Maker","Makes Blended Fruit Drinks");
      System.out.println("New product created successfully with id = "+id);
      Configuration.releaseRootApplicationModule(am,true);
    }
}
